# -*- coding: utf-8 -*-

from . import logger


# from base import reload_gs_profile
# from plone import api


def upgrade(setup_tool=None):
    """
    """
    logger.info("Running upgrade (Python): Add new field to settings")



# -*- coding: utf-8 -*-
from plone.app.contenttypes.testing import PLONE_APP_CONTENTTYPES_FIXTURE
from plone.app.robotframework.testing import REMOTE_LIBRARY_BUNDLE_FIXTURE
from plone.app.testing import applyProfile
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import PloneSandboxLayer
from plone.testing import z2

import collective.campaignmonitor


class CollectiveCampaignmonitorLayer(PloneSandboxLayer):

    defaultBases = (PLONE_APP_CONTENTTYPES_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        # Load any other ZCML that is required for your tests.
        # The z3c.autoinclude feature is disabled in the Plone fixture base
        # layer.
        self.loadZCML(package=collective.campaignmonitor)

    def setUpPloneSite(self, portal):
        applyProfile(portal, 'collective.campaignmonitor:default')


COLLECTIVE_CAMPAIGNMONITOR_FIXTURE = CollectiveCampaignmonitorLayer()


COLLECTIVE_CAMPAIGNMONITOR_INTEGRATION_TESTING = IntegrationTesting(
    bases=(COLLECTIVE_CAMPAIGNMONITOR_FIXTURE,),
    name='CollectiveCampaignmonitorLayer:IntegrationTesting',
)


COLLECTIVE_CAMPAIGNMONITOR_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(COLLECTIVE_CAMPAIGNMONITOR_FIXTURE,),
    name='CollectiveCampaignmonitorLayer:FunctionalTesting',
)


COLLECTIVE_CAMPAIGNMONITOR_ACCEPTANCE_TESTING = FunctionalTesting(
    bases=(
        COLLECTIVE_CAMPAIGNMONITOR_FIXTURE,
        REMOTE_LIBRARY_BUNDLE_FIXTURE,
        z2.ZSERVER_FIXTURE,
    ),
    name='CollectiveCampaignmonitorLayer:AcceptanceTesting',
)



# -*- coding: utf-8 -*-
from collective.campaignmonitor import _
from zope import schema
from zope.interface import Interface
from zope.publisher.interfaces.browser import IDefaultBrowserLayer

import re


class ICollectiveCampaignmonitorLayer(IDefaultBrowserLayer):
    """Marker interface that defines a browser layer."""


class ICampaignMonitorSettings(Interface):
    api_key = schema.TextLine(
        title=_(u"CampaignMonitor API Key"),
        description=_(
            u"help_api_key",
            default=u"Enter in your CampaignMonitor key here (.e.g. "
            + u"'8b785dcabe4b5aa24ef84201ea7dcded-us4'). Log into "
            + u"xxxx.createsend.com, go to account settings -> API Keys "
            + u"click on Show API key and copy the API Key to this field.",
        ),
        default=u"",
        required=True,
    )

    client_id = schema.Choice(
        title=_("Client ID"),
        description=_(
            u"Campaign Monitor uses clients to group lists, invoices, ... Select the one that you want to use in Plone"
        ),
        vocabulary="collective.campaignmonitor.CampaignMonitorClientsVocabulary",
        required=False,
    )

    force_resubscribe = schema.Bool(
        title=_("Force resubscription in default form?"),
        description=_(
            u"When adding a new subscriber from the API, Campaign Monitor checks if the user was previously unsubscribed from the list and if so it prevents from adding it. If this option is enabled, this check is bypassed. Be careful when using this option."
        ),
        default=False,
    )


class ICampaignMonitorConnection(Interface):
    def initialize():
        """ Load connection data from registy and prepare for serving results """

    def account():
        """ Return account information for campaign monitor """

    def clients():
        """ Return available clients at the campaign monitor account """

    def lists():
        """ Return available lists at the campaign monitor account """

    def subscribe(email, list_id):
        """ Subscribe 'email' to 'list_id' Campaign Monitor list """

    def list_details(list_id):
        """ get details about a given list """


class NotAnEmailAddress(schema.ValidationError):
    __doc__ = _(u"Invalid email address")


check_email = re.compile(r"[a-zA-Z0-9._%-]+@([a-zA-Z0-9-]+.)*[a-zA-Z]{2,4}").match


def validate_email(value):
    if not check_email(value):
        raise NotAnEmailAddress(value)
    return True


class INewsletterSubscribe(Interface):

    email = schema.TextLine(
        title=_(u"Email address"),
        description=_(u"help_email", default=u"Please enter your email address."),
        required=True,
        constraint=validate_email,
    )

    list_id = schema.Choice(
        title=_(u"List ID"),
        vocabulary="collective.campaignmonitor.CampaignMonitorListsVocabulary",
        required=False,
    )



# -*- coding: utf-8 -*-
from collective.campaignmonitor.testing import COLLECTIVE_CAMPAIGNMONITOR_INTEGRATION_TESTING  # noqa
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from zope.component import getUtility
from zope.schema.interfaces import IVocabularyFactory
from zope.schema.interfaces import IVocabularyTokenized

import unittest


class CampaignMonitorListsVocabularyIntegrationTest(unittest.TestCase):

    layer = COLLECTIVE_CAMPAIGNMONITOR_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])

    def test_vocab_campaign_monitor_lists_vocabulary(self):
        vocab_name = 'collective.campaignmonitor.CampaignMonitorListsVocabulary'
        factory = getUtility(IVocabularyFactory, vocab_name)
        self.assertTrue(IVocabularyFactory.providedBy(factory))

        vocabulary = factory(self.portal)
        self.assertTrue(IVocabularyTokenized.providedBy(vocabulary))
        # self.assertEqual(
        #     vocabulary.getTerm('sony-a7r-iii').title,
        #     _(u'Sony Aplha 7R III'),
        # )



# -*- coding: utf-8 -*-

# from plone import api
from collective.campaignmonitor.interfaces import ICampaignMonitorConnection
from zope.component import getUtility
from zope.interface import implementer
from zope.schema.interfaces import IVocabularyFactory
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary


@implementer(IVocabularyFactory)
class CampaignMonitorListsVocabulary(object):
    """
    """

    def __call__(self, context):
        connection = getUtility(ICampaignMonitorConnection)
        items = connection.lists()

        terms = []
        for item in items:
            terms.append(
                SimpleTerm(
                    value=item.get("list_id"),
                    token=str(item.get("list_id")),
                    title=item.get("list_name"),
                )
            )
        # Create a SimpleVocabulary from the terms list and return it:
        return SimpleVocabulary(terms)


CampaignMonitorListsVocabularyFactory = CampaignMonitorListsVocabulary()
